# Script for combining semantic data exported from CQPweb with data with syntactic patterns from the corpus and manual analysis

# Author: Barbara McGillivray
# Date: October 2016

# Import libraries:

import os
import csv
import xlrd

# Parameters:

verbs = ["give", "send", "lend", "offer", "sell", "show"]

# File and directory names:

directory = os.path.dirname(__file__)
directory_input_patterns = os.path.abspath(os.path.join(os.path.dirname( '__file__' ), '..', 'data', '[new]annotated data'))
directory_input_semantics = os.path.abspath(os.path.join(os.path.dirname( '__file__' ), '..', 'data', 'exported_data'))
directory_input_speakers = os.path.abspath(os.path.join(os.path.dirname( '__file__' ), '..', 'data'))
directory_output = os.path.abspath(os.path.join(os.path.dirname( '__file__' ), '..', 'data', 'combined_data'))
directory_logs = os.path.abspath(os.path.join(os.path.dirname( '__file__' ), '..', 'logs'))

for verb in verbs:

    file_in_patterns = verb + '_concordance_manually_annotated_complete.txt'
    file_in_semantics = verb + '_-55_metadata.txt'
    file_in_metadata = verb + '_50_kwic.txt'
    file_in_speakers = 'Spoken BNC2014 EAS metadata_cleaned.xlsx'
    file_out = verb + '_concordance_combined_data.txt'
    log_file = verb + ".log"

    # Define dictionaries:
    id2recipient = dict()
    id2theme = dict()
    id2all = dict()
    id2metadata = dict()
    speaker2data = dict()

    # Read input file with concordance data and manually analyzed patterns:
    header_file_in_patterns = list()
    count = 0
    with open(os.path.join(directory_input_patterns, file_in_patterns), 'rb') as csvfile:
        reader = csv.reader(csvfile, delimiter='\t', quotechar='|')
        for row in reader:
            count += 1
            if count == 1:
                header_file_in_patterns = row
            else:
                identifier = int(row[0])
                #if identifier == 7:
                print "Concordances: processing hit " + str(identifier) + " for verb " + verb
                print "Row:" + str(row)
                recipient = row[12]
                # if the recipient has more than one word we need to pick its headword:
                headword_recipient = row[14]
                headword_theme = row[15]
                #if ' ' in recipient:
                #    words_recipient = recipient.split(" ")
                #    if recipient.startswith('the '):
                #        recipient = words_recipient[1]
                #    else:
                #        recipient = words_recipient[len(words_recipient)-1]
                id2all[identifier] = row
                id2theme[identifier] = headword_theme
                id2recipient[identifier] = headword_recipient

    # Read metadata information from concordances:
    count = 0
    header_file_in_metadata = list()
    with open(os.path.join(directory_input_semantics, file_in_metadata), 'rb') as csvfile:
        reader = csv.reader(csvfile, delimiter='\t', quotechar='|')
        for row in reader:
            count += 1
            if count == 1:
                header_file_in_metadata = row
            else:
                identifier = int(row[0])
                #if identifier == 7:
                print "Metadata: processing hit " + str(identifier)
                date = row[8]
                num_speakers = row[10]
                location = row[12]
                speakers = row[13]
                relation = row[14]
                subject = row[15]
                topics = row[16]

                metadata = [date, num_speakers, location, speakers, relation, subject, topics]
                id2metadata[identifier] = metadata

    # Read input file with speakers' data:
    count = 0
    xl_workbook = xlrd.open_workbook(os.path.join(directory_input_speakers, file_in_speakers))
    xl_sheet = xl_workbook.sheet_by_name("Sheet1")
    header_file_in_speakers = [xl_sheet.cell(0, idx).value for idx in range(1, 25)]
    del header_file_in_speakers[19]
    for row_idx in range(1, xl_sheet.nrows):  # Iterate through rows
        print ('Row: %s' % row_idx)   # Print row number
        id = xl_sheet.cell(row_idx, 0).value
            # the line I'm reading has a valid speaker ID

        speaker_data = [xl_sheet.cell(row_idx, idx).value for idx in range(1, 25)]

        # age:
        try:
            speaker_data[0] = str(int(speaker_data[0]))
        except ValueError as e:
            print "Trying to read age"
            print id
            print speaker_data[0]
            print e
            speaker_data[0] = "NA"

        # no. of utterances
        try:
            speaker_data[22] = str(int(speaker_data[22]))
        except ValueError as e:
            print "Trying no. of utterances"
            print id
            print speaker_data[22]
            print e
            speaker_data[22] = "NA"

        # no. of words:
        try:
            speaker_data[23] = str(int(speaker_data[23]))
        except ValueError as e:
            print "Trying to read no. of words"
            print id
            print speaker_data[23]
            print e
            speaker_data[23] = "NA"

        del speaker_data[19] # this is the national security number and it creates problems because sometimes it's

        try:
            # a string and sometimes a float
            speaker2data[str(int(id))] = speaker_data
        except ValueError as e:
            print "Trying to read id"
            print id
            print e
            #id = "NA"


    print header_file_in_speakers

    # Open log file:
    log_file = open(os.path.join(directory_logs,log_file), "w")

    # Print combined output:

    with open(os.path.join(directory_output, file_out), 'wb') as out:
        writer = csv.writer(out, delimiter='\t', quotechar='|', quoting=csv.QUOTE_MINIMAL)
        #header:
        header = header_file_in_patterns + ['recipient_semantic_tag', 'theme_semantic_tag', 'verb_semantic_tag'] + ["date", "num_speakers", "location", "speakers", "relation", "subject", "topics"] + ['main_speaker_id'] + header_file_in_speakers
        writer.writerow(header)

        # open file with semantic class information:
        with open(os.path.join(directory_input_semantics, file_in_semantics), 'rb') as csvfile:
            reader = csv.reader(csvfile, delimiter='\t', quotechar='|')
            identifier = 0
            for row in reader:
                identifier = identifier + 1
                theme = ""
                recipient = ""
                if identifier in id2theme:
                    theme = id2theme[identifier]
                if identifier in id2recipient:
                    recipient = id2recipient[identifier]
                concordance = row[1]
                #print "concordance:",concordance
                semantics = row[2]
                speakers = row[3]
                words = concordance.split(' ')
                semantic_classes = semantics.split(' ')
                speaker_ids = speakers.split(' ')
                recipient_semantic = ""
                theme_semantic = ""
                verb_semantic = ""
                all = ""
                metadata = ""
                speaker_data = ""
                main_speaker_id = speaker_ids[5].lstrip('0').lstrip('0').lstrip('0')
                if main_speaker_id in speaker2data:
                    speaker_data = speaker2data[main_speaker_id]
                if identifier in id2all:
                    all = id2all[identifier]
                    verb_semantic = semantic_classes[5]
                if identifier in id2metadata:
                    metadata = id2metadata[identifier]

                for i in range(0,len(words)):
                    word = words[i]
                    #print "word", word
                    if ('ANON-name-f' in word and 'ANON-name-f' in theme) or ('ANON-name-m' in word and 'ANON-name-m' in theme) or ('ANON-place' in word and 'ANON-place' in theme):
                        theme_semantic = semantic_classes[i]
                    elif word == theme:
                        theme_semantic = semantic_classes[i]
                    elif ('ANON-name-f' in word and 'ANON-name-f' in recipient) or ('ANON-name-m' in word and 'ANON-name-m' in recipient) or ('ANON-place' in word and 'ANON-place' in recipient):
                        recipient_semantic = semantic_classes[i]
                    elif word == recipient:
                        recipient_semantic = semantic_classes[i]

                if all != '':
                    output_list = []
                    try:
                        all_out = '\t'.join(all)
                    except TypeError as e:
                        all_out = 'all', e
                    try:
                        metadata_out = '\t'.join(metadata)
                    except TypeError as e:
                        metadata_out = 'metadata', e
                    try:
                        speaker_out = '\t'.join(speaker_data)
                    except TypeError as e:
                        speaker_out = 'speaker', e
                    print "1 "+all_out
                    print "2 "+recipient_semantic
                    print "3 "+theme_semantic
                    print "4" +recipient_semantic
                    print "5 "+verb_semantic
                    print "6 "+metadata_out
                    print "7 "+main_speaker_id
                    print "8 "+speaker_out
                    try:
                        all_out = all_out.encode('utf-8')
                    except UnicodeDecodeError as e:
                        log_file.write("Error all_out:")
                        log_file.write(all_out)
                        log_file.write(str(e))
                        log_file.write("\n")
                    try:
                        recipient_semantic = recipient_semantic.encode('utf-8')
                    except UnicodeDecodeError as e:
                        log_file.write("Error recipient_semantic:")
                        log_file.write(recipient_semantic)
                        log_file.write(str(e))
                        log_file.write("\n")
                    try:
                        theme_semantic = theme_semantic.encode('utf-8')
                    except UnicodeDecodeError as e:
                        log_file.write("Error theme_semantic:")
                        log_file.write(theme_semantic)
                        log_file.write(str(e))
                        log_file.write("\n")
                    try:
                        verb_semantic = verb_semantic.encode('utf-8')
                    except UnicodeDecodeError as e:
                        log_file.write("Error verb_semantic:")
                        log_file.write(verb_semantic)
                        log_file.write(str(e))
                        log_file.write("\n")
                    try:
                        metadata_out = metadata_out.encode('utf-8')
                    except UnicodeDecodeError as e:
                        log_file.write("Error metadata_out:")
                        log_file.write(metadata_out)
                        log_file.write(str(e))
                        log_file.write("\n")
                    try:
                        speaker_out = speaker_out.encode('utf-8')
                    except UnicodeDecodeError as e:
                        log_file.write("Error speaker_out:")
                        log_file.write(speaker_out)
                        log_file.write(str(e))
                        log_file.write("\n")
                    output_list = [all_out, recipient_semantic, theme_semantic, verb_semantic, metadata_out,
                                   main_speaker_id, speaker_out]
                    try:
                        writer.writerow(output_list)
                    except TypeError as e:
                        print e
                        writer.writerow([identifier, e])
                #print '\t'.join(all),recipient_semantic, theme_semantic, verb_semantic,'\t'.join(metadata)

    log_file.close()